#!/bin/sh
#************************************************************************#
#* (c) Copyright 2008, ESE, UCI, USA                                    *#
#*                                                                      *#
#* This file is subject of copyright. All rights reserved.              *#
#* Further distribution and copying of the following information is     *#
#* only permitted with with explicit permission of the author.          *#
#*                                                                      *#
#* EXCLUSION OF WARRANTY:                                               *#
#* The file is distributed without any warranty.  No author or          *#
#* distributor accepts responsibility to anyone for the consequences of *#
#* using it or for whether it serves any particular purpose.            *#
#*                                                                      *#
#* File       : install.sh                                              *#
#* Author     : Yonghyun Hwang <yonghyuh@ics.uci.edu>                   *#
#* Date       : Thu Mar 27, 2008                                        *#
#************************************************************************#

VERSION="2.0"

# ---- functions 
check_requirement() {

    local GCC="gcc"
    local WHO="whoami"

    if ${WHO} | grep -c 'root' &> /dev/null ; then
        :
    else
        echo "ERROR: root priviliges required for ESE installation"
        exit 1
    fi

    if ${GCC} --version \
        | egrep 'GCC[^0-9]*(4\.[0-9]|3.[4-9])' >/dev/null 2>&1; then
        :
    else
        echo "ERROR: gcc version 3.4 or higher required for ESE installation"
        exit 1
    fi
}


# ---- main 

echo -e "Welcome to ESE Installation\n"

# check minimum requirements for ESE installation
check_requirement

# exit on error 
set -e

echo "==================================================================="
echo "Unzipping ESE archive"
echo "==================================================================="
tar xvzf ese-bin-$VERSION.tar.gz -C /

# fix group ownership for ese
chgrp -R users /opt/ese
chmod -R g+rX /opt/ese


echo "==================================================================="
echo "Unzipping bundled 3rd party archive"
echo "==================================================================="
tar xvzf ese-pkg-$VERSION.tar.gz -C /

# fix group ownership for pkg
chgrp -R users /opt/pkg
chmod -R g+rX /opt/ese

echo "Installation successful."
echo ""
echo "To start using ESE do:"
echo  "   source /opt/ese/bin/setup.sh"
echo  "   ese"
echo ""
echo "Thank you for using ESE"


