#ifndef RTOS_QUEUE_H
#define RTOS_QUEUE_H

#include <iostream>
#include <queue>
#include "prior_queue.h"

namespace ese_rtos {

  typedef enum { ROUND_ROBIN = 0, RR } SCHED_TYPE;

  template <class elemType>
    class TaskQueue {

  public:
    TaskQueue() {}
    virtual ~TaskQueue() {}

    virtual bool is_empty(void) = 0;
    virtual bool add_tail(const elemType & elem) = 0;
    virtual const elemType & get_head(void) = 0;
    virtual bool del_head(void) = 0;
    virtual void prt_content(void) = 0;
  };

  template <class elemType>
    class RRQueue : public TaskQueue<elemType> {

  public:
    RRQueue() {}
    virtual ~RRQueue() {}

    bool is_empty(void);
    bool add_tail(const elemType & elem);
    const elemType & get_head(void);
    bool del_head(void);
    void prt_content(void);

  private:
    std::queue<elemType> queue;
  };

  template <class elemType>
    bool RRQueue<elemType>::is_empty(void) {
    return queue.empty();
  }

  template <class elemType>
    bool RRQueue<elemType>::add_tail(const elemType & elem) {
    queue.push(elem);
    return true;
  }

  template <class elemType>
    const elemType & RRQueue<elemType>::get_head(void) {
    return queue.front();
  }

  template <class elemType>
    bool RRQueue<elemType>::del_head(void) {
    queue.pop();
    return 0;
  }

  template <class elemType>
    void RRQueue<elemType>::prt_content(void) {

    elemType elem;

    unsigned int size = queue.size();

    if(size == 0) {
      std::cout << "Queue is empty\n";
    }

    for(unsigned int i = 0; i < size; i++) {
      elem = queue.front();
      queue.pop();
      elem.prt_content();
      queue.push(elem);
    }
  }

  template <class elemType>
    class PriorQueue : public TaskQueue<elemType> {

  public:
    PriorQueue() {}
    virtual ~PriorQueue() {}

    bool is_empty(void);
    bool add_tail(const elemType & elem);
    const elemType & get_head(void);
    bool del_head(void);
    void prt_content(void);

  private:
    prior_queue<elemType> queue;
  };

  template <class elemType>
    bool PriorQueue<elemType>::is_empty(void) {
    return queue.empty();
  }

  template <class elemType>
    bool PriorQueue<elemType>::add_tail(const elemType & elem) {
    queue.push(elem);
    return true;
  }

  template <class elemType>
    const elemType & PriorQueue<elemType>::get_head(void) {
    return queue.top();
  }

  template <class elemType>
    bool PriorQueue<elemType>::del_head(void) {
    queue.pop();
    return 0;
  }

  template <class elemType>
    void PriorQueue<elemType>::prt_content(void) {

    elemType elem;

    unsigned int size = queue.size();

    if(size == 0) {
      std::cout << "Queue is empty\n";
    }

    for(unsigned int i = 0; i < size; i++) {
      elem = queue.front();
      queue.pop();
      elem.prt_content();
      queue.push_back(elem);
    }
  }



} // end of namespace ese_rtos

#endif // RTOS_QUEUE_H
