#ifndef RTOS_STATS_H
#define RTOS_STATS_H

#include <string>
#include "systemc.h"

//#include "rtos.h"

namespace ese_rtos {

  struct RTOS_Conf;
  class RTOS_Lat;

  class RTOS_Stats {

  public:
    RTOS_Stats(const RTOS_Conf & conf);
    ~RTOS_Stats();

    void wait_context_switch_lat(void);
    void wait_sched_lat(void);

    void wait_sem_p_lat(void);
    void wait_sem_v_lat(void);

    /** collect stats for PE idle time 
	CAUTION: set_idle_start and set_idle_end should always be paired!
     */
    /** Mark the time as a start of idle period */
    void set_idle_start(void);
    /** Mark the time as a end of idle period */
    void set_idle_end(void);
    /** accumulate idle time */
    void acc_idle_time(unsigned int cycles);

    void wait_isr_lat(void);
    void wait_ipc_send_lat(const unsigned int & numOfBytes);
    void wait_ipc_recv_lat(const unsigned int & numOfBytes);

    unsigned long long int get_idle_time(void);
    unsigned long long int get_overhead(void);

    //void do_check(void);
    void dump_rtos_statistics(const std::string & oFileName = "");

  public:
    /** this variable is solely for set_idle_start and set_idle_end */
    unsigned long long int idleStamp;

  private:
    std::string rtosType;
    RTOS_Lat * rtosLat;

    unsigned long int numOfSemP, numOfSemV, numOfContextSwitch, numOfIsr;

    unsigned long int numOfSend, numOfRecv, numOfSched;

    unsigned long long int sendTime, recvTime;
    unsigned long long int schedTime;
    unsigned long long int isrTime;

    unsigned long long int idleTime;

    void set_rtos_conf(const RTOS_Conf & conf);
  };

}
#endif
