#ifndef RTOS_THREAD_H
#define RTOS_THREAD_H

#include "rtos.h"

namespace ese_rtos {

  typedef enum { STATE_READY = 0, STATE_RUNNING, STATE_SLEEPING, STATE_WAITING, STATE_PENDING, STATE_INT_HANDLING, STATE_TERMINATED } TaskState;
  typedef enum { TYPE_PROCESS = 0 } TaskType;

  class i_ESE_RTOS;

  class ESE_Thread : public sc_module {

  public:
    SC_HAS_PROCESS(ESE_Thread);

    ESE_Thread(sc_module_name name, unsigned int pid, unsigned int priority, int (*_func)(void), void * param);

    /** Main function for ESE Thread */
    void main(void);

    /** Print information on ESE Thread. */
    void prt_info(void);

  public:
    int (*main_body) (void);
    void * funcParam;

    /** Info for Task Control */
    unsigned int pid;
    std::string name;
    unsigned int priority;

    TaskType type;
    TaskState state;

    /** remaining cycles that need to be consumes in next delta cycle */
    unsigned int cycles;

    sc_port<i_ESE_RTOS> rtos_call;
  };

  /** Initialization Thread.
      It starts scheduler to schedule the application threads.
      Before scheduler is running, all the application threads are pending.
      This thread doesn't need TCB because it just makes scheduler run. */
  class ESE_Init_Thread : public sc_module {

  public:
    SC_HAS_PROCESS(ESE_Init_Thread);

    ESE_Init_Thread(sc_module_name name);

    void main();
    void prt_info(void);

  public:
    sc_port<i_ESE_RTOS> rtos_call;
  };

} // end of namespace ese_rtos

#endif //RTOS_THREAD_H
