/*
 * libmad - MPEG audio decoder library
 * Copyright (C) 2000-2004 Underbit Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: decoder.h,v 1.1.1.1 2008/11/06 00:12:50 ese Exp $
 */

# ifndef LIBMAD_DECODER_H
# define LIBMAD_DECODER_H

# include "stream.h"
# include "frame.h"
# include "synth.h"
//PC: Needed to write to file
#ifdef DEBUG_INC_LIBS
//# include <stdio.h>
#endif

enum mad_decoder_mode {
  MAD_DECODER_MODE_SYNC  = 0,
  MAD_DECODER_MODE_ASYNC
};

enum mad_flow {
  MAD_FLOW_CONTINUE = 0x0000,	/* continue normally */
  MAD_FLOW_STOP     = 0x0010,	/* stop decoding normally */
  MAD_FLOW_BREAK    = 0x0011,	/* stop decoding and signal an error */
  MAD_FLOW_IGNORE   = 0x0020	/* ignore the current frame */
};

struct mad_decoder {
  enum mad_decoder_mode mode;

  int options;

  struct {
    ln pid;
    int inp;
    int outp;
  } async;

  //PC: Inorder to replace malloc to static allocation
  struct syncdata {
    struct mad_stream stream;
    struct mad_frame frame;
    struct mad_synth synth;
  }syncobj; 

  struct syncdata *sync;

  void *cb_data;
};
void mad_decoder_init(struct mad_decoder *, void *);

int mad_decoder_finish(struct mad_decoder *);

# define mad_decoder_options(decoder, opts)  \
    ((void) ((decoder)->options = (opts)))

//PC: Slight changes in the function params
#if 0
int mad_decoder_run(struct mad_decoder *, enum mad_decoder_mode);
#endif
#ifdef DEBUG_INC_LIBS
 // __declspec( dllexport )
int mad_decoder_run(unsigned char const *start, uln length, unsigned char *outfile);
#else
 // __declspec( dllexport )
int mad_decoder_run(unsigned char const *start, uln length);
#endif

int mad_decoder_message(struct mad_decoder *, void *, unsigned int *);

# endif
